@@ -1,13 +1,14 @@
 use bevy::{
-    app::{Events, ManualEventReader},
-    ecs::schedule::SystemSet,
+    ecs::{
+        event::{Events, ManualEventReader},
+        schedule::SystemSet,
+    },
     input::mouse::{MouseMotion, MouseWheel},
     prelude::*,
     render::{
         camera::Camera,
         camera::CameraProjection,
-        camera::{ActiveCameras, PerspectiveProjection},
-        render_graph::base::camera::CAMERA_3D,
+        camera::{ActiveCamera, Camera3d, PerspectiveProjection},
     },
     window::Windows,
 };
@@ -15,21 +16,6 @@ use bevy::{
 use strum::IntoEnumIterator;
 use strum_macros::EnumIter;
 
-/*
-pub trait NextEnum: IntoEnumIterator {
-    fn next_enum<E>(pred: S)
-    where
-    E: IntoEnumIterator,
-    S: ResMut<State<E>>,
-    {
-        let a = E::iter().enumerate();
-        a.nth(
-            a.find(|a| a.1 == *pred.current())
-        )
-    }
-}
-*/
-
 #[macro_export]
 macro_rules! next_enum {
     ($l:ident, $k:expr) => {
@@ -53,6 +39,7 @@ macro_rules! next_enum {
     };
 }
 
+#[derive(Component)]
 pub struct PlayerMove;
 #[derive(Clone, Eq, PartialEq, Debug, Hash)]
 enum PluginState {
@@ -131,24 +118,27 @@ impl Default for PlayerSettings {
     }
 }
 
+#[derive(Debug, Hash, PartialEq, Eq, Clone, SystemLabel)]
+struct MovementUpdate;
+
 pub struct ConfigCam;
 impl Plugin for ConfigCam {
-    fn build(&self, app: &mut AppBuilder) {
+    fn build(&self, app: &mut App) {
         app.init_resource::<CamLogic>()
             .add_plugin(NoCameraPlayerPlugin)
             .init_resource::<PlayerSettings>()
             .add_state(PluginState::Enabled)
-            .add_state(CameraState::LookAt)
+            .add_state(CameraState::FollowBehind)
             .add_state(ScrollType::MovementSpeed)
-            .add_system(toggle_camera_parent.system().after("move"))
-            .add_system(switch_scroll_type.system().after("move"))
-            .add_system(scroll.system().after("move"))
-            .add_system(cycle_cam_state.system().after("move"))
-            .add_system_set(SystemSet::on_enter(PluginState::Enabled).with_system(setup.system()))
+            .add_system(toggle_camera_parent.after(MovementUpdate))
+            .add_system(switch_scroll_type.after(MovementUpdate))
+            .add_system(scroll.after(MovementUpdate))
+            .add_system(cycle_cam_state.after(MovementUpdate))
+            .add_system_set(SystemSet::on_enter(PluginState::Enabled).with_system(setup))
             .add_system_set(
                 SystemSet::on_update(PluginState::Enabled)
-                    .with_system(move_player.system().after("move"))
-                    .with_system(move_camera.system().label("move")),
+                    .with_system(move_player.after(MovementUpdate))
+                    .with_system(move_camera.label(MovementUpdate)),
             );
     }
 }
@@ -188,7 +178,7 @@ fn cycle_cam_state(
 fn setup(
     mut commands: Commands,
     asset_server: Res<AssetServer>,
-    mut cl: ResMut<CamLogic>,
+    mut _cl: ResMut<CamLogic>,
     settings: Res<PlayerSettings>,
     mut meshes: ResMut<Assets<Mesh>>,
     mut materials: ResMut<Assets<StandardMaterial>>,
@@ -219,15 +209,11 @@ fn setup(
         })
     };
 
-    cl.player.entity = Some(
+    _cl.player.entity = Some(
         b.insert(PlayerMove)
             .with_children(|parent| {
                 parent
                     .spawn_bundle(PerspectiveCameraBundle {
-                        camera: Camera {
-                            name: Some("player".to_string()),
-                            ..Default::default()
-                        },
                         transform: Transform::from_xyz(-2.0, 5.0, 5.0)
                             .looking_at(Vec3::ZERO, Vec3::Y),
                         ..Default::default()
@@ -241,7 +227,6 @@ fn setup(
     commands
         .spawn_bundle(PerspectiveCameraBundle {
             camera: Camera {
-                name: Some("Camera3d".to_string()),
                 ..Default::default()
             },
             transform: Transform::from_xyz(-2.0, 5.0, 5.0).looking_at(Vec3::ZERO, Vec3::Y),
@@ -322,7 +307,7 @@ fn move_camera(
     state: Res<State<CameraState>>,
     mut cl: ResMut<CamLogic>,
     mut settings: ResMut<MovementSettings>,
-    mut transforms: QuerySet<(Query<(&mut Transform, &Camera)>, Query<&Transform>)>,
+    mut transforms: ParamSet<(Query<&mut Transform, With<Camera3d>>, Query<&Transform>)>,
 ) {
     let mut delta_trans = Transform::identity();
     settings.disable_look = true;
@@ -337,9 +322,10 @@ fn move_camera(
         CameraState::LookAt => {
             // if there is both a player and a bonus, target the mid-point of them
             if let (Some(player_entity), Some(bonus_entity)) = (cl.player.entity, cl.target) {
+                let transform_query = transforms.p1();
                 if let (Ok(player_transform), Ok(bonus_transform)) = (
-                    transforms.q1().get(player_entity),
-                    transforms.q1().get(bonus_entity),
+                    transform_query.get(player_entity),
+                    transform_query.get(bonus_entity),
                 ) {
                     cl.camera_should_focus = player_transform
                         .translation
@@ -347,7 +333,7 @@ fn move_camera(
                 }
             // otherwise, if there is only a player, target the player
             } else if let Some(player_entity) = cl.player.entity {
-                if let Ok(player_transform) = transforms.q1().get(player_entity) {
+                if let Ok(player_transform) = transforms.p1().get(player_entity) {
                     cl.camera_should_focus = player_transform.translation;
                 }
             // otherwise, target the middle
@@ -357,7 +343,7 @@ fn move_camera(
         }
         _ => {
             if let Some(player_entity) = cl.player.entity {
-                if let Ok(player_transform) = transforms.q1().get(player_entity) {
+                if let Ok(player_transform) = transforms.p1().get(player_entity) {
                     match *state.current() {
                         CameraState::Fps => {
                             delta_trans.translation = player_transform.translation;
@@ -415,49 +401,31 @@ fn move_camera(
         cl.camera_is_focus += camera_motion;
     }
     // look at that new camera's actual focus
-    for (mut transform, camera) in transforms.q0_mut().iter_mut() {
-        if camera.name == Some(CAMERA_3D.to_string()) {
-            if delta_trans.translation != Vec3::ZERO {
-                *transform = delta_trans
-            } else {
-                *transform = transform.looking_at(cl.camera_is_focus, Vec3::Y)
-            }
+    for mut transform in transforms.p0().iter_mut() {
+        if delta_trans.translation != Vec3::ZERO {
+            *transform = delta_trans
+        } else {
+            *transform = transform.looking_at(cl.camera_is_focus, Vec3::Y)
         }
     }
 }
 
 #[allow(clippy::type_complexity)]
 fn toggle_camera_parent(
-    mut act_cams: ResMut<ActiveCameras>,
+    mut act_cams: ResMut<ActiveCamera<Camera3d>>,
     mut settings: ResMut<MovementSettings>,
-    mut query: QuerySet<(
-        Query<(&FlyCam, &mut Camera)>,
-        Query<(&PlayerCam, &mut Camera)>,
-    )>,
+    mut query: ParamSet<(Query<(&FlyCam, Entity)>, Query<(&PlayerCam, Entity)>)>,
 ) {
     if settings.locked_to_player && !settings.ltp {
-        act_cams.remove("Camera3d");
-
-        let (_, mut b) = query.q1_mut().single_mut().unwrap();
-        b.name = Some("Camera3d".to_string());
-
-        act_cams.add("Camera3d");
-
-        let (_, mut b) = query.q0_mut().single_mut().unwrap();
-        b.name = Some("Test".to_string());
+        let p1 = query.p1();
+        let (_, b) = p1.single();
+        act_cams.set(b);
 
         settings.ltp = true;
     } else if !settings.locked_to_player && settings.ltp {
-        act_cams.remove("Camera3d");
-
-        let (_, mut b) = query.q0_mut().single_mut().unwrap();
-        b.name = Some("Camera3d".to_string());
-
-        act_cams.add("Camera3d");
-
-        let (_, mut b) = query.q1_mut().single_mut().unwrap();
-        b.name = Some("Test".to_string());
-
+        let pr = query.p0();
+        let (_, b) = pr.single();
+        act_cams.set(b);
         settings.ltp = false;
     }
 }
@@ -592,7 +560,10 @@ impl Default for MovementSettings {
 }
 
 /// Used in queries when you want flycams and not other cameras
+
+#[derive(Component)]
 struct FlyCam;
+#[derive(Component)]
 struct PlayerCam;
 
 /// Grabs/ungrabs mouse cursor
@@ -684,18 +655,21 @@ fn player_look(
         return;
     }
     let window = windows.get_primary().unwrap();
+    let mut state_delta = state.as_mut();
     for (_camera, mut transform) in query.iter_mut() {
-        for ev in state.reader_motion.iter(&motion) {
+        for ev in state_delta.reader_motion.iter(&motion) {
             if window.cursor_locked() {
-                state.pitch -= (settings.sensitivity * ev.delta.y * window.height()).to_radians();
-                state.yaw -= (settings.sensitivity * ev.delta.x * window.width()).to_radians();
+                state_delta.pitch -=
+                    (settings.sensitivity * ev.delta.y * window.height()).to_radians();
+                state_delta.yaw -=
+                    (settings.sensitivity * ev.delta.x * window.width()).to_radians();
             }
 
-            state.pitch = state.pitch.clamp(-1.54, 1.54);
+            state_delta.pitch = state_delta.pitch.clamp(-1.54, 1.54);
 
             // Order is important to prevent unintended roll
-            transform.rotation = Quat::from_axis_angle(Vec3::Y, state.yaw)
-                * Quat::from_axis_angle(Vec3::X, state.pitch);
+            transform.rotation = Quat::from_axis_angle(Vec3::Y, state_delta.yaw)
+                * Quat::from_axis_angle(Vec3::X, state_delta.pitch);
         }
     }
 }
@@ -710,26 +684,26 @@ fn cursor_grab(keys: Res<Input<KeyCode>>, mut windows: ResMut<Windows>) {
 /// Contains everything needed to add first-person fly camera behavior to your game
 pub struct PlayerPlugin;
 impl Plugin for PlayerPlugin {
-    fn build(&self, app: &mut AppBuilder) {
+    fn build(&self, app: &mut App) {
         app.init_resource::<InputState>()
             .init_resource::<MovementSettings>()
-            .add_startup_system(setup_player.system())
-            .add_startup_system(initial_grab_cursor.system())
-            .add_system(player_move.system().before("move"))
-            .add_system(player_look.system().after("move"))
-            .add_system(cursor_grab.system().after("move"));
+            .add_startup_system(setup_player)
+            .add_startup_system(initial_grab_cursor)
+            .add_system(player_move.before(MovementUpdate))
+            .add_system(player_look.after(MovementUpdate))
+            .add_system(cursor_grab.after(MovementUpdate));
     }
 }
 
 /// Same as `PlayerPlugin` but does not spawn a camera
 pub struct NoCameraPlayerPlugin;
 impl Plugin for NoCameraPlayerPlugin {
-    fn build(&self, app: &mut AppBuilder) {
+    fn build(&self, app: &mut App) {
         app.init_resource::<InputState>()
             .init_resource::<MovementSettings>()
-            .add_startup_system(initial_grab_cursor.system())
-            .add_system(player_move.system().before("move"))
-            .add_system(player_look.system().after("move"))
-            .add_system(cursor_grab.system().after("move"));
+            .add_startup_system(initial_grab_cursor)
+            .add_system(player_move.before(MovementUpdate))
+            .add_system(player_look.after(MovementUpdate))
+            .add_system(cursor_grab.after(MovementUpdate));
     }
 }
